package com.docus.server.visitor;

import com.docus.core.util.Func;
import com.docus.core.util.json.JSON;
import com.docus.log.context.TrackContext;
import com.docus.log.processor.AbstractProcessor;
import com.docus.server.collect.infrastructure.dao.CollectTypeEnum;
import com.docus.server.collect.infrastructure.dao.StateEnum;
import com.docus.server.collect.infrastructure.enums.IIntegerEnum;
import com.docus.server.tool.ParamsUtils;
import com.docus.server.tool.SpringUtils;
import com.docus.server.ws.ITaskOriginalMessageService;
import com.docus.server.ws.IWsResult;
import com.docus.server.ws.convert.IConverter;
import lombok.extern.slf4j.Slf4j;

import java.util.Map;

/**
 * @author linruifeng
 */
@Slf4j
public class VisitorProcessor extends AbstractProcessor {
    private IConverter converter;
    private IWsResult wsResult;
    private ITaskOriginalMessageService taskOriginalMessageService;

    @Override
    public Object doProcess(TrackContext context) {
        if (context.isError()) {
            return afterThrowingProcess(context);
        } else {
            return afterReturnProcess(context);
        }
    }

    /**
     * 前置通知
     */
    @Override
    public Object beforeProcess(TrackContext context) {
        log.debug("=== AOP 前置通知 ===");
        initBeans(context.getBeanNames());
        String message = (String) context.getArgs()[0];
        if (Func.isEmpty(message)) {
            throw new RuntimeException("参数为空");
        }
        Object dto = converter.convert(message, context.getMethodName());
        String jsonStr = JSON.toJSON(dto);
        SpringUtils.invokeMethod(context.getClassType(), "setProperty", new Object[]{jsonStr});
        Long taskId = taskOriginalMessageService.insertTaskOriginalMessage(jsonStr, message,
                IIntegerEnum.fromDisplay(CollectTypeEnum.class, context.getGroup()));
        return ParamsUtils.addParam("taskId", taskId).addParam("jsonStr", jsonStr).param();
    }


    /**
     * 后置通知
     */
    public Object afterReturnProcess(TrackContext context) {
        log.debug("=== AOP 后置通知 ===");
        Map beforeResult = (Map) context.getBeforeResult();
        Long taskId = (Long) beforeResult.get("taskId");
        String afterReturnResult = (String) beforeResult.get("jsonStr");
        if (Func.isNotEmpty(taskId)) {
            taskOriginalMessageService.updateTaskOriginalMessage(taskId, afterReturnResult, StateEnum.OK);
        }
        Map map = JSON.fromJSON(afterReturnResult, Map.class);
        map.put("msg", "操作成功！");
        return wsResult.ok(map);
    }

    /**
     * 异常通知
     */
    public Object afterThrowingProcess(TrackContext context) {
        log.error("=== AOP 异常通知 ===");
        Map<String, Object> params = ParamsUtils.addParam("msg", context.getExMessageResult()).param();
        try {
            Map beforeResult = (Map) context.getBeforeResult();
            Long taskId = (Long) beforeResult.get("taskId");
            String afterReturnResult = (String) beforeResult.get("jsonStr");

            log.error(context.getExMessageResult());
            if (Func.isNotEmpty(taskId)) {
                taskOriginalMessageService.updateTaskOriginalMessage(taskId, afterReturnResult, StateEnum.FAIL);
            }
            params.putAll(JSON.fromJSON(afterReturnResult, Map.class));
            return wsResult.fail(params);
        } catch (Exception e) {
            return wsResult.fail(params);
        }
    }

    /**
     * 最后通知
     */
    @Override
    public Object afterProcess(TrackContext context) {
        log.debug("=== AOP 最后通知 ===");
        return null;
    }

    private void initBeans(String[] beanNames) {
        this.converter = (IConverter) SpringUtils.getBean(beanNames[0]);
        this.wsResult = (IWsResult) SpringUtils.getBean(beanNames[1]);
        this.taskOriginalMessageService = SpringUtils.getBean(ITaskOriginalMessageService.class);
    }
}
